@@ -30,11 +30,21 @@ namespace arrow {
 namespace compute {
 
 static std::shared_ptr<DataType> GetOffsetType(const DataType& type) {
-  return type.id() == Type::LIST ? int32() : int64();
+  switch (type.id()) {
+    case Type::LIST:
+    case Type::LIST_VIEW:
+      return int32();
+    case Type::LARGE_LIST:
+    case Type::LARGE_LIST_VIEW:
+      return int64();
+    default:
+      Unreachable("Unexpected type");
+  }
 }
 
 TEST(TestScalarNested, ListValueLength) {
-  for (auto ty : {list(int32()), large_list(int32())}) {
+  for (auto ty : {list(int32()), large_list(int32()), list_view(int32()),
+                  large_list_view(int32())}) {
     CheckScalarUnary("list_value_length", ty, "[[0, null, 1], null, [2, 3], []]",
                      GetOffsetType(*ty), "[3, null, 2, 0]");
   }
@@ -47,7 +57,8 @@ TEST(TestScalarNested, ListValueLength) {
 TEST(TestScalarNested, ListElementNonFixedListWithNulls) {
   auto sample = "[[7, 5, 81], [6, null, 4, 7, 8], [3, 12, 2, 0], [1, 9], null]";
   for (auto ty : NumericTypes()) {
-    for (auto list_type : {list(ty), large_list(ty)}) {
+    for (auto list_type :
+         {list(ty), large_list(ty), list_view(ty), large_list_view(ty)}) {
       auto input = ArrayFromJSON(list_type, sample);
       auto null_input = ArrayFromJSON(list_type, "[null]");
       for (auto index_type : IntTypes()) {
@@ -117,33 +128,45 @@ TEST(TestScalarNested, ListElementInvalid) {
               Raises(StatusCode::Invalid));
 }
 
+using VarLenListLikeTypeFactory =
+    std::shared_ptr<DataType> (*)(std::shared_ptr<DataType>);
+static const VarLenListLikeTypeFactory kVarLenListTypeFactories[] = {
+    list,
+    large_list,
+    list_view,
+    large_list_view,
+};
+
 TEST(TestScalarNested, ListSliceVariableOutput) {
   const auto value_types = {float32(), int32()};
   for (auto value_type : value_types) {
-    auto input = ArrayFromJSON(list(value_type), "[[1, 2, 3], [4, 5], [6], null]");
-    ListSliceOptions args(/*start=*/0, /*stop=*/2, /*step=*/1,
-                          /*return_fixed_size_list=*/false);
-    auto expected = ArrayFromJSON(list(value_type), "[[1, 2], [4, 5], [6], null]");
-    CheckScalarUnary("list_slice", input, expected, &args);
+    for (auto list_type_factory : kVarLenListTypeFactories) {
+      ListSliceOptions args(/*start=*/0, /*stop=*/2, /*step=*/1,
+                            /*return_fixed_size_list=*/false);
+      auto list_ty = list_type_factory(value_type);
+      auto input = ArrayFromJSON(list_ty, "[[1, 2, 3], [4, 5], [6], null]");
+      auto expected = ArrayFromJSON(list_ty, "[[1, 2], [4, 5], [6], null]");
+      CheckScalarUnary("list_slice", input, expected, &args);
 
-    args.start = 1;
-    expected = ArrayFromJSON(list(value_type), "[[2], [5], [], null]");
-    CheckScalarUnary("list_slice", input, expected, &args);
+      args.start = 1;
+      expected = ArrayFromJSON(list_ty, "[[2], [5], [], null]");
+      CheckScalarUnary("list_slice", input, expected, &args);
 
-    args.start = 2;
-    args.stop = 4;
-    expected = ArrayFromJSON(list(value_type), "[[3], [], [], null]");
-    CheckScalarUnary("list_slice", input, expected, &args);
+      args.start = 2;
+      args.stop = 4;
+      expected = ArrayFromJSON(list_ty, "[[3], [], [], null]");
+      CheckScalarUnary("list_slice", input, expected, &args);
 
-    args.start = 1;
-    args.stop = std::nullopt;
-    expected = ArrayFromJSON(list(value_type), "[[2, 3], [5], [], null]");
-    CheckScalarUnary("list_slice", input, expected, &args);
+      args.start = 1;
+      args.stop = std::nullopt;
+      expected = ArrayFromJSON(list_ty, "[[2, 3], [5], [], null]");
+      CheckScalarUnary("list_slice", input, expected, &args);
 
-    args.start = 0;
-    args.stop = 4;
-    args.step = 2;
-    expected = ArrayFromJSON(list(value_type), "[[1, 3], [4], [6], null]");
+      args.start = 0;
+      args.stop = 4;
+      args.step = 2;
+      expected = ArrayFromJSON(list_ty, "[[1, 3], [4], [6], null]");
+    }
   }
 
   // Verify passing `return_fixed_size_list=false` with fixed size input
@@ -158,9 +181,13 @@ TEST(TestScalarNested, ListSliceVariableOutput) {
 TEST(TestScalarNested, ListSliceFixedOutput) {
   const auto value_types = {float32(), int32()};
   for (auto value_type : value_types) {
-    auto inputs = {ArrayFromJSON(list(value_type), "[[1, 2, 3], [4, 5], [6], null]"),
-                   ArrayFromJSON(fixed_size_list(value_type, 3),
-                                 "[[1, 2, 3], [4, 5, null], [6, null, null], null]")};
+    const char* kVarLenListJSON = "[[1, 2, 3], [4, 5], [6], null]";
+    const char* kFixedSizeListJSON = "[[1, 2, 3], [4, 5, null], [6, null, null], null]";
+    std::vector<std::shared_ptr<Array>> inputs;
+    for (auto list_type_factory : kVarLenListTypeFactories) {
+      inputs.push_back(ArrayFromJSON(list_type_factory(value_type), kVarLenListJSON));
+    }
+    inputs.push_back(ArrayFromJSON(fixed_size_list(value_type, 3), kFixedSizeListJSON));
     for (auto input : inputs) {
       ListSliceOptions args(/*start=*/0, /*stop=*/2, /*step=*/1,
                             /*return_fixed_size_list=*/true);
@@ -187,7 +214,7 @@ TEST(TestScalarNested, ListSliceFixedOutput) {
         CheckScalarUnary("list_slice", input, expected, &args);
       } else {
         EXPECT_RAISES_WITH_MESSAGE_THAT(
-            NotImplemented,
+            Invalid,
             ::testing::HasSubstr("Unable to produce FixedSizeListArray from "
                                  "non-FixedSizeListArray without `stop` being set."),
             CallFunction("list_slice", {input}, &args));
@@ -253,22 +280,25 @@ TEST(TestScalarNested, ListSliceChildArrayOffset) {
   ASSERT_EQ(input->offset(), 0);
   ASSERT_EQ(input->values()->offset(), 2);
 
-  ListSliceOptions args(/*start=*/0, /*stop=*/2, /*step=*/1,
+  ListSliceOptions args(/*start=*/0, /*stop=*/3, /*step=*/1,
                         /*return_fixed_size_list=*/false);
   auto expected = ArrayFromJSON(list(int8()), "[[2], [3, 4]]");
   CheckScalarUnary("list_slice", input, expected, &args);
 
   args.return_fixed_size_list = true;
-  expected = ArrayFromJSON(fixed_size_list(int8(), 2), "[[2, null], [3, 4]]");
+  expected = ArrayFromJSON(fixed_size_list(int8(), 3), "[[2, null, null], [3, 4, null]]");
   CheckScalarUnary("list_slice", input, expected, &args);
 }
 
 TEST(TestScalarNested, ListSliceOutputEqualsInputType) {
+  const char* kVarLenListJSON = "[[1, 2, 3], [4, 5], [6, null], null]";
+  const char* kFixedLenListJSON = "[[1, 2], [4, 5], [6, null], null]";
   // Default is to return same type as the one passed in.
-  auto inputs = {
-      ArrayFromJSON(list(int8()), "[[1, 2, 3], [4, 5], [6, null], null]"),
-      ArrayFromJSON(large_list(int8()), "[[1, 2, 3], [4, 5], [6, null], null]"),
-      ArrayFromJSON(fixed_size_list(int8(), 2), "[[1, 2], [4, 5], [6, null], null]")};
+  std::vector<std::shared_ptr<Array>> inputs;
+  for (auto list_type_factory : kVarLenListTypeFactories) {
+    inputs.push_back(ArrayFromJSON(list_type_factory(int8()), kVarLenListJSON));
+  }
+  inputs.push_back(ArrayFromJSON(fixed_size_list(int8(), 2), kFixedLenListJSON));
   for (auto input : inputs) {
     ListSliceOptions args(/*start=*/0, /*stop=*/2, /*step=*/1);
     auto expected = ArrayFromJSON(input->type(), "[[1, 2], [4, 5], [6, null], null]");
@@ -305,10 +335,9 @@ TEST(TestScalarNested, ListSliceBadParameters) {
   // stop not set and FixedSizeList requested with variable sized input
   args.stop = std::nullopt;
   EXPECT_RAISES_WITH_MESSAGE_THAT(
-      NotImplemented,
-      ::testing::HasSubstr("NotImplemented: Unable to produce FixedSizeListArray from "
-                           "non-FixedSizeListArray without "
-                           "`stop` being set."),
+      Invalid,
+      ::testing::HasSubstr("Invalid: Unable to produce FixedSizeListArray from "
+                           "non-FixedSizeListArray without `stop` being set."),
       CallFunction("list_slice", {input}, &args));
   // Catch step must be >= 1
   args.start = 0;
